==> vaycore-0.5.2.tar.gz: lib/http/net_request.dart <==
import 'package:vaycore/dio/default_adapter.dart';
import 'package:vaycore/http.dart';

///
/// @author vay
/// @email hxhvay@126.com
/// @date 2020-05-09
///

/// 解析器
typedef Parser<M> = M Function<M>(dynamic data);

/// 网络请求对象
class NetRequest {
  final BaseNetAdapter _adapter;
  List<dynamic> _interceptors;
  final Parser _parser;

  NetRequest({
    BaseNetAdapter adapter,
    List<dynamic> interceptors,
    Parser parser,
  })  : this._adapter = adapter ?? DefaultAdapter(),
        this._parser = parser,
        this._interceptors = interceptors;

  List<dynamic> get interceptors {
    if (_interceptors == null) {
      _interceptors = List();
    }
    return this._interceptors;
  }

  BaseNetAdapter get adapter => _adapter;

  /// 网络请求
  Future<HttpResult<T>> request<T>(String url,
      {HttpMethod method = HttpMethod.GET,
      int timeout = 15000,
      String contentType = "application/json; charset=utf-8",
      dynamic responseType,
      Map<String, dynamic> data,
      Map<String, dynamic> header,
      int retryCount = 3,
      dynamic transformer,
      List<dynamic> interceptors,
      Parser<T> parser}) {
    assert(url != null);
    assert(_adapter != null);
    parser ??= this._parser;
    assert(parser != null);
    try {
      // 拦截器处理
      if (interceptors == null) {
        interceptors = List.from(this.interceptors);
      } else {
        interceptors?.insertAll(0, this.interceptors);
      }

      // 请求交给adapter处理
      return _adapter.request<T>(RequestParams.builder()
          .setUrl(url)
          .setMethod(method)
          .setHeader(header)
          .setTimeout(timeout)
          .setData(data)
          .setResponseType(responseType)
          .setTransformer(transformer)
          .setInterceptors(interceptors)
          .setParser(parser)
          .build());
    } catch (e) {
      return Future.value(HttpResult<T>(ResultStatus.intercept, error: e));
    }
  }
}

/// 网络请求适配器
abstract class BaseNetAdapter {
  /// 核心请求方法
  Future<HttpResult<T>> request<T>(RequestParams params);

  /// 将参数合并到url
  String wrapUrlByParams(
      HttpMethod method, String url, Map<String, dynamic> params) {
    // 只有GET请求、HEAD请求才会把参数拼接到url
    if (method != HttpMethod.GET && method != HttpMethod.HEAD) {
      return url;
    }
    if (url.contains("?")) {
      url = _handleUrlParams(url, params);
    }
    if (params != null && params.isNotEmpty) {
      StringBuffer sb = new StringBuffer();
      params.forEach((key, value) {
        if (sb.isEmpty) {
          sb.write("$url?");
        } else {
          sb.write("&");
        }
        sb.write("$key=$value");
      });
      return sb.toString();
    }
    return url;
  }

  /// 处理url原有的参数，如果包含参数，则返回的url会自动移除url追加的参数
  String _handleUrlParams(String url, Map<String, dynamic> params) {
    if (params == null || params.isEmpty) {
      return url;
    }
    Map<String, String> urlParams = Uri.parse(url)?.queryParameters;
    if (urlParams != null && urlParams.isNotEmpty) {
      urlParams.forEach((key, value) {
        if (key == null || key.isEmpty || value == null || value.isEmpty) {
          return;
        }
        if (!params.containsKey(key)) {
          params[key] = value;
        }
      });
      return url.split("?")[0];
    }
    return url;
  }
}

/// 请求结果类
class HttpResult<T> {
  final ResultStatus status;
  final int httpStatus;
  final T data;
  final dynamic error;

  HttpResult(
    this.status, {
    this.data,
    this.error,
    this.httpStatus = 0,
  });
}

==> vaycore-0.5.4.tar.gz: lib/http/net_request.dart <==
import 'package:vaycore/dio/default_adapter.dart';
import 'package:vaycore/http.dart';

///
/// @author vay
/// @email hxhvay@126.com
/// @date 2020-05-09
///

/// 解析器
typedef Parser = M Function<M>(dynamic data);

/// 网络请求对象
class NetRequest {
  final BaseNetAdapter _adapter;
  List<dynamic> _interceptors;
  final Parser parser;

  NetRequest({
    BaseNetAdapter adapter,
    List<dynamic> interceptors,
    Parser parser,
  })  : this._adapter = adapter ?? DefaultAdapter(),
        this.parser = parser,
        this._interceptors = interceptors;

  List<dynamic> get interceptors {
    if (_interceptors == null) {
      _interceptors = List();
    }
    return this._interceptors;
  }

  BaseNetAdapter get adapter => _adapter;

  /// 网络请求
  Future<HttpResult<T>> request<T>(String url,
      {HttpMethod method = HttpMethod.GET,
      int timeout = 15000,
      String contentType = "application/json; charset=utf-8",
      dynamic responseType,
      Map<String, dynamic> data,
      Map<String, dynamic> header,
      int retryCount = 3,
      dynamic transformer,
      List<dynamic> interceptors,
      Parser parser}) {
    assert(url != null);
    assert(_adapter != null);
    // 传入的泛型不是dynamic、Map、List类型，则解析器不能为空
    if (T.toString() != "dynamic" &&
        T.toString() != "Map<dynamic, dynamic>" &&
        T.toString() != "List<dynamic>") {
      parser ??= this.parser;
      assert(parser != null);
    } else {
      // 传入的泛型是dynamic、Map、List类型, 解析器需要为空
      assert(parser == null);
    }
    try {
      // 拦截器处理
      if (interceptors == null) {
        interceptors = List.from(this.interceptors);
      } else {
        interceptors?.insertAll(0, this.interceptors);
      }

      // 请求交给adapter处理
      return _adapter.request<T>(RequestParams.builder()
          .setUrl(url)
          .setMethod(method)
          .setHeader(header)
          .setTimeout(timeout)
          .setData(data)
          .setResponseType(responseType)
          .setTransformer(transformer)
          .setInterceptors(interceptors)
          .setParser(parser)
          .build());
    } catch (e) {
      return Future.value(HttpResult<T>(ResultStatus.intercept, error: e));
    }
  }
}

/// 网络请求适配器
abstract class BaseNetAdapter {
  /// 核心请求方法
  Future<HttpResult<T>> request<T>(RequestParams params);

  /// 将参数合并到url
  String wrapUrlByParams(
      HttpMethod method, String url, Map<String, dynamic> params) {
    // 只有GET请求、HEAD请求才会把参数拼接到url
    if (method != HttpMethod.GET && method != HttpMethod.HEAD) {
      return url;
    }
    if (url.contains("?")) {
      url = _handleUrlParams(url, params);
    }
    if (params != null && params.isNotEmpty) {
      StringBuffer sb = new StringBuffer();
      params.forEach((key, value) {
        if (sb.isEmpty) {
          sb.write("$url?");
        } else {
          sb.write("&");
        }
        sb.write("$key=$value");
      });
      return sb.toString();
    }
    return url;
  }

  /// 处理url原有的参数，如果包含参数，则返回的url会自动移除url追加的参数
  String _handleUrlParams(String url, Map<String, dynamic> params) {
    if (params == null || params.isEmpty) {
      return url;
    }
    Map<String, String> urlParams = Uri.parse(url)?.queryParameters;
    if (urlParams != null && urlParams.isNotEmpty) {
      urlParams.forEach((key, value) {
        if (key == null || key.isEmpty || value == null || value.isEmpty) {
          return;
        }
        if (!params.containsKey(key)) {
          params[key] = value;
        }
      });
      return url.split("?")[0];
    }
    return url;
  }
}

/// 请求结果类
class HttpResult<T> {
  final ResultStatus status;
  final int httpStatus;
  final T data;
  final dynamic error;

  HttpResult(
    this.status, {
    this.httpStatus = 0,
    this.data,
    this.error,
  });

  HttpResult<T> copyWith<T>({
    ResultStatus status,
    int httpStatus,
    T data,
    dynamic error,
  }) {
    return HttpResult<T>(
      status ?? this.status,
      httpStatus: httpStatus ?? this.httpStatus,
      data: data ?? this.data,
      error: error ?? this.error,
    );
  }
}
